//
// seriously modified ZJ Wood January 2015 - conversion to glfw
// inclusion of matrix stack Feb. 2015
// original from Shinjiro Sueda
// October, 2014
//

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <cassert>
#include <cmath>
#include <stdio.h>
#include "GLSL.h"
#include "tiny_obj_loader.h"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr
#include "RenderingHelper.h"
#include "ngraph/ngraph.hpp"



GLFWwindow* window;
using namespace std;
using namespace NGraph;

vector<tinyobj::shape_t> shapes;
vector<tinyobj::material_t> materials;

vector<tinyobj::shape_t> shapes2;
vector<tinyobj::material_t> materials2;

GLuint prog;
GLuint posBufObj = 0;
GLuint norBufObj = 0;
GLuint indBufObj = 0;

GLuint posBufObj2 = 0;
GLuint norBufObj2 = 0;
GLuint indBufObj2 = 0;

GLint aPos = 0;
GLint aNor = 0;
GLint uMV = 0;
GLint uP = 0;
GLuint rotationTheta = 0;
GLuint viewVectorID = 0;
GLuint innerID = 0;
GLuint uViewMatrix = 0;

static float g_width, g_height;


float theta = -M_PI/2.0;
float phi = 0.0;

glm::vec3 eye = glm::vec3(0.0, 0.0, 0.0);
glm::vec3 up = glm::vec3(0.0, 1.0, 0.0);
glm::vec3 lookAt = glm::vec3(0.0, 0.0, -1.0);

glm::mat4 view = glm::mat4(1);

Graph nodes;

const int signalsSize = 50;
const int nodeSize = 6;

struct Sphere
{
	float x;
	float y;
	float z;
	float rot;
	float time;
	//max time is + .75
};

struct Node
{
	float x;
	float y;
	float z;
	float rot;
	bool react;
	Sphere signals[signalsSize];

	float minX;
	float maxX;
	float minY;
	float maxY;
};

//creates null sphere
Sphere createNullSphere()
{
	Sphere temp;
	temp.x = -1000.0;
	temp.y = -1000.0;
	temp.z = -1000.0;
	temp.rot = -1000.0;
	temp.time = -1000.0;
	return temp;
}



Node refNode[nodeSize];

Sphere nullSphere = createNullSphere();


//declare a matrix stack
RenderingHelper ModelTrans;

//set up NullSphere
bool equalsNullSphere(Sphere sp)
{
	return
		sp.x == nullSphere.x &&
		sp.y == nullSphere.y &&
		sp.z == nullSphere.z &&
		sp.rot == nullSphere.rot &&
		sp.time == nullSphere.time;
}


//converts degrees to radians
float degToRad(int deg)
{
	return deg * M_PI / 180.0;
}

//creates struct
Node createNode(float x, float y, float z, float rot, bool react)
{
	Node temp;
	temp.x = x;
	temp.y = y;
	temp.z = z;
	temp.rot = rot;
	temp.react = react;

	for (int i = 0; i < signalsSize; i++)
	{
		temp.signals[i] = nullSphere;
	}

	float radRot = degToRad(rot);
	temp.minX = (x + (-.7 * cos(radRot))) * (g_width/2);
	temp.minY = ((y + (-.7 * sin(radRot))) - .15) * (g_height/2);
	temp.maxX = (x + (-.2 * cos(radRot))) * (g_width/2);
	temp.maxY = ((y + (-.2 * sin(radRot))) + .15) * (g_height/2);

	return temp;
}

//initializes Sphere values to given node values
Sphere createSphere(Node node)
{
	Sphere temp;
	float radRot = degToRad(node.rot);
	temp.x = node.x + (-.5 * cos(radRot));
	temp.y = node.y + (-.5 * sin(radRot)) - .026;
	temp.z = node.z;
	temp.rot = node.rot;
	temp.time = 0.0;
	return temp;
}

//updates min-max coords of neurons
void updateMinMax(int ver, float shiftX, float shiftY)
{
	if (shiftX < 0.0)
	{
		refNode[ver].minX += shiftX * (g_width/2);
		refNode[ver].maxX += shiftX * (g_width/2);

		/*for (int i = 0; i < signalsSize; i++)
		{
			if (!equalsNullSphere(refNode[ver].signals[i]))
			{
				refNode[ver].signals[i].rot -= (refNode[ver].rot / 100);
			}

		}*/



	}
	else
	{
		refNode[ver].minX += shiftX * (g_width/2);
		refNode[ver].maxX += shiftX * (g_width/2);


		/*for (int i = 0; i < signalsSize; i++)
		{
			if (!equalsNullSphere(refNode[ver].signals[i]))
				refNode[ver].signals[i].rot += (refNode[ver].rot / 100);
		}*/

	}

}


//Given a vector of shapes which has already been read from an obj file
// resize all vertices to the range [-1, 1]
void resize_obj(std::vector<tinyobj::shape_t> &shapes){
    float minX, minY, minZ;
    float maxX, maxY, maxZ;
    float scaleX, scaleY, scaleZ;
    float shiftX, shiftY, shiftZ;
    float epsilon = 0.001;

    minX = minY = minZ = 1.1754E+38F;
    maxX = maxY = maxZ = -1.1754E+38F;

    //Go through all vertices to determine min and max of each dimension
    for (size_t i = 0; i < shapes.size(); i++) {
        for (size_t v = 0; v < shapes[i].mesh.positions.size() / 3; v++) {
            if(shapes[i].mesh.positions[3*v+0] < minX) minX = shapes[i].mesh.positions[3*v+0];
            if(shapes[i].mesh.positions[3*v+0] > maxX) maxX = shapes[i].mesh.positions[3*v+0];

            if(shapes[i].mesh.positions[3*v+1] < minY) minY = shapes[i].mesh.positions[3*v+1];
            if(shapes[i].mesh.positions[3*v+1] > maxY) maxY = shapes[i].mesh.positions[3*v+1];

            if(shapes[i].mesh.positions[3*v+2] < minZ) minZ = shapes[i].mesh.positions[3*v+2];
            if(shapes[i].mesh.positions[3*v+2] > maxZ) maxZ = shapes[i].mesh.positions[3*v+2];
        }
    }
   //From min and max compute necessary scale and shift for each dimension
   float maxExtent, xExtent, yExtent, zExtent;
   xExtent = maxX-minX;
   yExtent = maxY-minY;
   zExtent = maxZ-minZ;
   if (xExtent >= yExtent && xExtent >= zExtent) {
     maxExtent = xExtent;
   }
   if (yExtent >= xExtent && yExtent >= zExtent) {
     maxExtent = yExtent;
   }
   if (zExtent >= xExtent && zExtent >= yExtent) {
     maxExtent = zExtent;
   }
    scaleX = 2.0 /maxExtent;
    shiftX = minX + (xExtent/ 2.0);
    scaleY = 2.0 / maxExtent;
    shiftY = minY + (yExtent / 2.0);
    scaleZ = 2.0/ maxExtent;
    shiftZ = minZ + (zExtent)/2.0;

    //Go through all verticies shift and scale them
    for (size_t i = 0; i < shapes.size(); i++) {
        for (size_t v = 0; v < shapes[i].mesh.positions.size() / 3; v++) {
            shapes[i].mesh.positions[3*v+0] = (shapes[i].mesh.positions[3*v+0] - shiftX) * scaleX;
            assert(shapes[i].mesh.positions[3*v+0] >= -1.0 - epsilon);
            assert(shapes[i].mesh.positions[3*v+0] <= 1.0 + epsilon);
            shapes[i].mesh.positions[3*v+1] = (shapes[i].mesh.positions[3*v+1] - shiftY) * scaleY;
            assert(shapes[i].mesh.positions[3*v+1] >= -1.0 - epsilon);
            assert(shapes[i].mesh.positions[3*v+1] <= 1.0 + epsilon);
            shapes[i].mesh.positions[3*v+2] = (shapes[i].mesh.positions[3*v+2] - shiftZ) * scaleZ;
            assert(shapes[i].mesh.positions[3*v+2] >= -1.0 - epsilon);
            assert(shapes[i].mesh.positions[3*v+2] <= 1.0 + epsilon);
        }
    }
}



void loadShapes(const string &objFile)
{
	string err = tinyobj::LoadObj(shapes, materials, objFile.c_str());
	if(!err.empty()) {
		cerr << err << endl;
	}
   resize_obj(shapes);
}

void loadShapes2(const string &objFile)
{
	string err = tinyobj::LoadObj(shapes2, materials2, objFile.c_str());
	if(!err.empty()) {
		cerr << err << endl;
	}
   resize_obj(shapes2);
}

void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods)
{
	double speed = .1;

    if (key == GLFW_KEY_A && action == GLFW_PRESS)
    {
    	glm::vec3 viewV = glm::normalize(lookAt - eye);
    	glm::vec3 strafe = glm::normalize(glm::cross(viewV, up));
    	eye.x -= strafe.x * speed;
    	lookAt.x -= strafe.x * speed;
    	/*eye.y -= strafe.y * speed;
    	lookAt.y -= strafe.y * speed;
    	eye.z -= strafe.z * speed;
    	lookAt.z -= strafe.z * speed;*/

    	for (int i = 0; i < nodeSize; i++)
    	{
    		updateMinMax(i, strafe.x*speed, strafe.y*speed);
    	}
    	//printf("shift: %f %f\n", -strafe.x*speed, -strafe.y*speed);
    }
    if (key == GLFW_KEY_D && action == GLFW_PRESS)
    {
    	glm::vec3 viewV = glm::normalize(lookAt - eye);
    	glm::vec3 strafe = glm::normalize(glm::cross(viewV, up));
    	eye.x += strafe.x * speed;
    	lookAt.x += strafe.x * speed;
    	/*eye.y += strafe.y * speed;
    	lookAt.y += strafe.y * speed;
    	eye.z += strafe.z * speed;
    	lookAt.z += strafe.z * speed;*/

    	for (int i = 0; i < nodeSize; i++)
    	{
    		updateMinMax(i, -strafe.x*speed, -strafe.y*speed);
    	}
    }
 


}

void update()
{
	view = glm::lookAt(eye, lookAt, up);
}


void mouse_callback(GLFWwindow *win, int button, int actions, int mods)
{
	double tempX, tempY;
	glfwGetCursorPos(window, &tempX, &tempY);
	tempX = (tempX - g_width/2);
	tempY = -1 * (tempY - g_height/2);

	//printf("Cursor x: %f, cursor y: %f\n", tempX, tempY);
	for (int i = 0; i < nodeSize; i++)
	{
		if (tempX >= refNode[i].minX &&
			tempX <= refNode[i].maxX &&
			tempY >= refNode[i].minY &&
			tempY <= refNode[i].maxY)
		{
			int temp = 0;
	        while (!equalsNullSphere(refNode[i].signals[temp]))
	        {
	        	temp++;
	        }
	        if (temp < signalsSize)
	        {
	        	refNode[i].signals[temp] = createSphere(refNode[i]);
	        	refNode[i].react = true;
	        }
	        else
	       	{
	       		cout << "Can't add any more spheres\n";
	       	}
		}
	}
}


void initGL()
{
	// Set the background color
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	// Enable Z-buffer test
	glEnable(GL_DEPTH_TEST);
	
	// Send the position array to the GPU
	const vector<float> &posBuf = shapes[0].mesh.positions;
	glGenBuffers(1, &posBufObj);
	glBindBuffer(GL_ARRAY_BUFFER, posBufObj);
	glBufferData(GL_ARRAY_BUFFER, posBuf.size()*sizeof(float), &posBuf[0], GL_STATIC_DRAW);
	
	// Send the normal array to the GPU
	const vector<float> &norBuf = shapes[0].mesh.normals;
	glGenBuffers(1, &norBufObj);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObj);
	glBufferData(GL_ARRAY_BUFFER, norBuf.size()*sizeof(float), &norBuf[0], GL_STATIC_DRAW);
	
	// Send the index array to the GPU
	const vector<unsigned int> &indBuf = shapes[0].mesh.indices;
	glGenBuffers(1, &indBufObj);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObj);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indBuf.size()*sizeof(unsigned int), &indBuf[0], GL_STATIC_DRAW);

	// Unbind the arrays
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	GLSL::checkVersion();
	assert(glGetError() == GL_NO_ERROR);

	// Send the position2 array to the GPU
	const vector<float> &posBuf2 = shapes2[0].mesh.positions;
	glGenBuffers(1, &posBufObj2);
	glBindBuffer(GL_ARRAY_BUFFER, posBufObj2);
	glBufferData(GL_ARRAY_BUFFER, posBuf2.size()*sizeof(float), &posBuf2[0], GL_STATIC_DRAW);
	
	// Send the normal2 array to the GPU
	const vector<float> &norBuf2 = shapes2[0].mesh.normals;
	glGenBuffers(1, &norBufObj2);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObj2);
	glBufferData(GL_ARRAY_BUFFER, norBuf2.size()*sizeof(float), &norBuf2[0], GL_STATIC_DRAW);

	// Send the index2 array to the GPU
	const vector<unsigned int> &indBuf2 = shapes2[0].mesh.indices;
	glGenBuffers(1, &indBufObj2);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObj2);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indBuf2.size()*sizeof(unsigned int), &indBuf2[0], GL_STATIC_DRAW);

	// Unbind the arrays
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	GLSL::checkVersion();
	assert(glGetError() == GL_NO_ERROR);

   //initialize the modeltrans matrix stack
    ModelTrans.useModelViewMatrix();
    ModelTrans.loadIdentity();
    

   //initialize graph
    nodes.insert_edge(0, 1);
    nodes.insert_edge(0, 2);
    nodes.insert_edge(1, 4);
    nodes.insert_edge(2, 3);
    nodes.insert_edge(5, 0);

    refNode[0] = createNode(-.5, 0.0, -2.0, 0.0, true);
    refNode[0].signals[0] = createSphere(refNode[0]);

    refNode[1] = createNode(.7, -.408, -2.0, -25.0, true);
    refNode[1].signals[0] = createSphere(refNode[1]);

    refNode[2] = createNode(.7, .392, -2.0, 25.0, true);
    refNode[2].signals[0] = createSphere(refNode[2]);

    refNode[3] = createNode(2.0, .6, -2.0, -20.0, false);

    refNode[4] = createNode(2.0, -.54, -2.0, -10.0, false);

    refNode[5] = createNode(-1.8, 0.0, -2.0, 0.0, false);

    int radius = 4;

    lookAt.x = radius*cos(phi)*cos(theta);
	lookAt.y = radius*sin(phi);
	lookAt.z = radius*cos(phi)*cos((M_PI/2.0) - theta);

    update();
}

bool installShaders(const string &vShaderName, const string &fShaderName)
{
	GLint rc;
	
	// Create shader handles
	GLuint VS = glCreateShader(GL_VERTEX_SHADER);
	GLuint FS = glCreateShader(GL_FRAGMENT_SHADER);
	
	// Read shader sources
	const char *vshader = GLSL::textFileRead(vShaderName.c_str());
	const char *fshader = GLSL::textFileRead(fShaderName.c_str());
	glShaderSource(VS, 1, &vshader, NULL);
	glShaderSource(FS, 1, &fshader, NULL);
	
	// Compile vertex shader
	glCompileShader(VS);
	GLSL::printError();
	glGetShaderiv(VS, GL_COMPILE_STATUS, &rc);
	GLSL::printShaderInfoLog(VS);
	if(!rc) {
		printf("Error compiling vertex shader %s\n", vShaderName.c_str());
		return false;
	}
	
	// Compile fragment shader
	glCompileShader(FS);
	GLSL::printError();
	glGetShaderiv(FS, GL_COMPILE_STATUS, &rc);
	GLSL::printShaderInfoLog(FS);
	if(!rc) {
		printf("Error compiling fragment shader %s\n", fShaderName.c_str());
		return false;
	}
	
	// Create the program and link
	prog = glCreateProgram();
	glAttachShader(prog, VS);
	glAttachShader(prog, FS);
	glLinkProgram(prog);
	GLSL::printError();
	glGetProgramiv(prog, GL_LINK_STATUS, &rc);
	GLSL::printProgramInfoLog(prog);
	if(!rc) {
		printf("Error linking shaders %s and %s\n", vShaderName.c_str(), fShaderName.c_str());
		return false;
	}
	
	// Set up the shader variables
	aPos = GLSL::getAttribLocation(prog, "aPos");
	aNor = GLSL::getAttribLocation(prog, "aNor");
	uMV = GLSL::getUniformLocation(prog, "MV");
	uP = GLSL::getUniformLocation(prog, "P");
	rotationTheta = GLSL::getUniformLocation(prog, "rTheta");
	viewVectorID = GLSL::getUniformLocation(prog, "viewVec");
	innerID = GLSL::getUniformLocation(prog, "inner");
	uViewMatrix = GLSL::getUniformLocation(prog, "viewMatrix");
	
	assert(glGetError() == GL_NO_ERROR);
	return true;
}

//draws electrical signals
void drawSpheres(Node *node, int nIndices, int vertex)
{
	for (int i = 0; i < signalsSize; i++)
	{
		if (!equalsNullSphere(node->signals[i]))
		{
			float radRot = degToRad(node->signals[i].rot);
			float tempTime = node->signals[i].time;
			Sphere tempSphere = node->signals[i];

			glUniform1f(innerID, 1.0);
			ModelTrans.loadIdentity();
   			ModelTrans.translate(glm::vec3(tempSphere.x + (tempTime * cos(radRot)),
   				 tempSphere.y + (tempTime * sin(radRot)), 
   				 tempSphere.z));

   			

   			ModelTrans.scale(.005, .005, .005);
			glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
			glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);

			node->signals[i].time = node->signals[i].time + .001;
			if (node->signals[i].time > .39  && node->signals[i].time < .391 && vertex != 5)
			{
				node->signals[i].rot -= (node->signals[i].rot / 20);
				//cout << "nailed it\n";
			}
			if (node->signals[i].time > .59  && node->signals[i].time < .591  && vertex != 5)
			{
				node->signals[i].rot -= (node->signals[i].rot / 20);
				//cout << "nailed it\n";
			}
			//printf("time = %f\n", node->signals[i].time);*/
		}
	}
}

void setNewSignals(int ver)
{
	Graph::vertex_set So = nodes.out_neighbors(ver);

	// print out each out-going edge
    //
    for (Graph::vertex_set::const_iterator t = So.begin(); t !=So.end(); t++)
    {
        //cout << *t << "\n";
        int temp = 0;
        while (!equalsNullSphere(refNode[*t].signals[temp]))
        {
        	temp++;
        }
        if (temp < signalsSize)
        {
        	refNode[*t].signals[temp] = createSphere(refNode[*t]);
        	refNode[*t].react = true;
        }
        else
       	{
       		cout << "Can't add any more spheres\n";
       	}
    }


}

//handles end case
void isEnd(int vertex)
{
	int anyLeft = 0;
	for (int i = 0; i < signalsSize; i++)
	{
		if (!equalsNullSphere(refNode[vertex].signals[i]))
		{
			if (refNode[vertex].signals[i].time >= .95)
			{
				refNode[vertex].signals[i] = nullSphere;
				setNewSignals(vertex);

				//cout << "time > .85" << "\n";
			}
			else
			{
				anyLeft += 1;
				//cout << "incrementing anyleft\n";
			}
		}
	}

	if (anyLeft == 0)
	{
		refNode[vertex].react = false;
		//printf("Now false: %d\n", vertex);
	}
}


void drawGL()
{
	// Clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//enable transparency
	glEnable (GL_BLEND); 
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	// Use our GLSL program
	glUseProgram(prog);

		// Compute and send the projection matrix - leave this as is
   glm::mat4 Projection = glm::perspective(45.0f, (float)g_width/g_height, 0.1f, 100.f); 
	glUniformMatrix4fv(uP, 1, GL_FALSE, glm::value_ptr(Projection));

	//send view vector and rotation to GPU
	glUniform3f(viewVectorID, 0.0, 0.0, 4.0);
	glm::mat4 rota = glm::mat4(1);
	rota = glm::rotate(theta, vec3(0.0, 1.0, 0.0));
	glUniformMatrix4fv(rotationTheta, 1, GL_FALSE, glm::value_ptr(rota));

	//set up view matrix
	update();
	glUniformMatrix4fv(uViewMatrix, 1, GL_FALSE, glm::value_ptr(view));

	//SPHERES

	// Enable and bind position array for drawing
	GLSL::enableVertexAttribArray(aPos);
	glBindBuffer(GL_ARRAY_BUFFER, posBufObj2);
	glVertexAttribPointer(aPos, 3, GL_FLOAT, GL_FALSE, 0, 0);
	
	// Enable and bind normal array for drawing
	GLSL::enableVertexAttribArray(aNor);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObj2);
	glVertexAttribPointer(aNor, 3, GL_FLOAT, GL_FALSE, 0, 0);
	
	// Bind index array for drawing
	int nIndices = (int)shapes2[0].mesh.indices.size();
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObj2);

	if (refNode[0].react)
	{
		drawSpheres(&refNode[0], nIndices, 0);
		isEnd(0);

	}

	if (refNode[1].react)
	{
		drawSpheres(&refNode[1], nIndices, 1);
		isEnd(1);

	}

	if (refNode[2].react)
	{
		drawSpheres(&refNode[2], nIndices, 2);
		isEnd(2);

	}

	if (refNode[3].react)
	{
		drawSpheres(&refNode[3], nIndices, 3);
		isEnd(3);
	}

	if (refNode[4].react)
	{
		drawSpheres(&refNode[4], nIndices, 4);
		isEnd(4);
	}

	if (refNode[5].react)
	{
		drawSpheres(&refNode[5], nIndices, 5);
		isEnd(5);
	}




	// Disable and unbind
	GLSL::disableVertexAttribArray(aPos);
	GLSL::disableVertexAttribArray(aNor);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	// Enable and bind position array for drawing
	GLSL::enableVertexAttribArray(aPos);
	glBindBuffer(GL_ARRAY_BUFFER, posBufObj);
	glVertexAttribPointer(aPos, 3, GL_FLOAT, GL_FALSE, 0, 0);
	
	// Enable and bind normal array for drawing
	GLSL::enableVertexAttribArray(aNor);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObj);
	glVertexAttribPointer(aNor, 3, GL_FLOAT, GL_FALSE, 0, 0);
	
	// Bind index array for drawing
	nIndices = (int)shapes[0].mesh.indices.size();
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObj);

	
 
   //create the model transforms 
	float scaleFactor = .7;

	//nueron 0
	glUniform1f(innerID, 0.0);
   	ModelTrans.loadIdentity();
   	ModelTrans.translate(glm::vec3(refNode[0].x, refNode[0].y, refNode[0].z));
   	ModelTrans.rotate(refNode[0].rot, glm::vec3(0, 0, 1));
   	ModelTrans.rotate(-90, glm::vec3(0, 1, 0));
   	ModelTrans.scale(scaleFactor, scaleFactor, scaleFactor);
	glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);

	



	//nueron 1
	glUniform1f(innerID, 0.0);
   	ModelTrans.loadIdentity();
   	ModelTrans.translate(glm::vec3(.7, -.4, -2.0));
   	ModelTrans.rotate(-25, glm::vec3(0, 0, 1));
   	ModelTrans.rotate(-90, glm::vec3(0, 1, 0)); 	
   	ModelTrans.scale(scaleFactor, scaleFactor, scaleFactor);
	glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);


	//nueron 2
	glUniform1f(innerID, 0.0);
   	ModelTrans.loadIdentity();
   	ModelTrans.translate(glm::vec3(refNode[2].x, .4, refNode[2].z));
   	ModelTrans.rotate(refNode[2].rot, glm::vec3(0, 0, 1));
   	ModelTrans.rotate(-90, glm::vec3(0, 1, 0));
   	ModelTrans.scale(scaleFactor, scaleFactor, scaleFactor);
	glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);

	//nueron 3
	glUniform1f(innerID, 0.0);
   	ModelTrans.loadIdentity();
   	ModelTrans.translate(glm::vec3(refNode[3].x, .62, refNode[3].z));
   	ModelTrans.rotate(-10, glm::vec3(0, 0, 1));
   	ModelTrans.rotate(-90, glm::vec3(0, 1, 0));
   	ModelTrans.scale(scaleFactor, scaleFactor, scaleFactor);
	glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);

	//nueron 4
	glUniform1f(innerID, 0.0);
   	ModelTrans.loadIdentity();
   	ModelTrans.translate(glm::vec3(refNode[4].x, -.52, refNode[4].z));
   	ModelTrans.rotate(-10.0, glm::vec3(0, 0, 1));
   	ModelTrans.rotate(-90, glm::vec3(0, 1, 0));
   	ModelTrans.scale(scaleFactor, scaleFactor, scaleFactor);
	glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);

	//neuron 5
	glUniform1f(innerID, 0.0);
   	ModelTrans.loadIdentity();
   	ModelTrans.translate(glm::vec3(refNode[5].x, refNode[5].y, refNode[5].z));
   	ModelTrans.rotate(refNode[5].rot, glm::vec3(0, 0, 1));
   	ModelTrans.rotate(-90, glm::vec3(0, 1, 0));
   	ModelTrans.scale(scaleFactor, scaleFactor, scaleFactor);
	glUniformMatrix4fv(uMV, 1, GL_FALSE, glm::value_ptr(ModelTrans.modelViewMatrix));
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);
	

   // Disable and unbind
	GLSL::disableVertexAttribArray(aPos);
	GLSL::disableVertexAttribArray(aNor);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	


	glUseProgram(0);
	assert(glGetError() == GL_NO_ERROR);
	
}



void reshapeGL(int w, int h)
{
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	g_width = w;
	g_height = h;
}


int main(int argc, char **argv)
{

   // Initialise GLFW
   if( !glfwInit() ) {
        fprintf( stderr, "Failed to initialize GLFW\n" );
        return -1;
   }

   glfwWindowHint(GLFW_SAMPLES, 4);
   glfwWindowHint(GLFW_RESIZABLE,GL_FALSE);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 2);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 1);

   g_width = 1024;
   g_height = 768;
   // Open a window and create its OpenGL context
   window = glfwCreateWindow( g_width, g_height, "lab 6 - HM", NULL, NULL);
   if( window == NULL ){
      fprintf( stderr, "Failed to open GLFW window. If you have an Intel GPU, they are not 3.3 compatible. Try the 2.1 version of the tutorials.\n" );
        glfwTerminate();
        return -1;
    }
    glfwMakeContextCurrent(window);

   // Ensure we can capture the escape key being pressed below
   glfwSetInputMode(window, GLFW_STICKY_KEYS, GL_TRUE);

   glfwSetKeyCallback(window, key_callback);
   glfwSetMouseButtonCallback(window, mouse_callback);

	loadShapes("neuron.obj");
	loadShapes2("sphere2.obj");

   std::cout << " loaded the object " << endl;

   // Initialize glad
   if (!gladLoadGL()) {
      fprintf(stderr, "Unable to initialize glad");
      glfwDestroyWindow(window);
      glfwTerminate();
      return 1;
   }

	initGL();
	installShaders("vert.glsl", "frag.glsl");

    do{
      drawGL();
        // Swap buffers
      glfwSwapBuffers(window);
      glfwPollEvents();

    } // Check if the ESC key was pressed or the window was closed
   while( glfwGetKey(window, GLFW_KEY_ESCAPE ) != GLFW_PRESS &&
         glfwWindowShouldClose(window) == 0 );

    // Close OpenGL window and terminate GLFW
    glfwTerminate();

	return 0;
}
